<?php

/**
 * DeltaV Plugin
 *
 * This plugin provides a html representation, so that a WebDAV server may be accessed
 * using a browser.
 *
 * The class intercepts GET requests to collection resources and generates a simple 
 * html index. 
 * 
 * @package Sabre
 * @subpackage DAV
 * @copyright Copyright (C) 2007-2011 Rooftop Solutions. All rights reserved.
 * @author Evert Pot (http://www.rooftopsolutions.nl/)
 * @license http://code.google.com/p/sabredav/wiki/License Modified BSD License
 */
class Sabre_DeltaV_Plugin extends Sabre_DAV_ServerPlugin {

    /**
     * reference to server class 
     * 
     * @var Sabre_DAV_Server 
     */
    protected $server;

    /**
     * enableEditing
     * 
     * @var bool 
     */
    protected $autoVersion = true;
	
	protected $deltaVBackend;
	
    /**
     * Creates the object.
     *
     * By default it will allow file creation and uploads.
     * Specify the first argument as false to disable this
     * 
     * @param bool $enablePost 
     * @return void
     */
    public function __construct($backend, $autoVersion=true) {

		// Required (similar to the server), Repository "Collection" (require an implementation of, "ICollection, IProperty and INode)
		// 			The "IProperty" must require and Abstact class, whos block property updating
	
        $this->autoVersion = $autoVersion; 
		
		$this->deltaVBackend = $deltaVBackend;

    }

    /**
     * Initializes the plugin and subscribes to events 
     * 
     * @param Sabre_DAV_Server $server 
     * @return void
     */
    public function initialize(Sabre_DAV_Server $server) {

        $this->server = $server;
		
		
		
        $this->server->subscribeEvent('beforeMethod',array($this,'httpGetInterceptor'));
		
		// TODO: Listen to "Lock" and "Unlock" event (see 3.2.2)
		
		// Subscribe to Unknow Method
        $this->server->subscribeEvent('unknownMethod',array($this,'unknownMethodHandler'));
    }

    /**
     * This method intercepts GET requests to collections and returns the html 
     * 
     * @param string $method 
     * @return bool 
     */
    public function httpGetInterceptor($method, $uri) {

        if ($method!='GET') return true;

        try { 
            $node = $this->server->tree->getNodeForPath($uri);
        } catch (Sabre_DAV_Exception_FileNotFound $e) {
            // We're simply stopping when the file isn't found to not interfere 
            // with other plugins.
            return;
        }
        if ($node instanceof Sabre_DAV_IFile) 
            return;

        $this->server->httpResponse->sendStatus(200);
        $this->server->httpResponse->setHeader('Content-Type','text/html; charset=utf-8');

        $this->server->httpResponse->sendBody(
            $this->generateDirectoryIndex($uri)
        );

        return false;
        
    }

    /**
     * Handles POST requests for tree operations
     * 
     * This method is not yet used.
     * 
     * @param string $method 
     * @return bool
     */
    public function unknownMethodHandler($method, $uri) {

        
        switch(strtoupper($method)) {

            case 'VERSION-CONTROL' :
				
				
				
				
                break;
            case 'put' :
                if ($_FILES) $file = current($_FILES);
                else break;
                $newName = trim($file['name']);
                list(, $newName) = Sabre_DAV_URLUtil::splitPath(trim($file['name']));
                if (isset($_POST['name']) && trim($_POST['name']))
                    $newName = trim($_POST['name']);

                // Making sure we only have a 'basename' component
                list(, $newName) = Sabre_DAV_URLUtil::splitPath($newName);
                    
               
                if (is_uploaded_file($file['tmp_name'])) {
                    $parent = $this->server->tree->getNodeForPath(trim($uri,'/'));
                    $parent->createFile($newName,fopen($file['tmp_name'],'r'));
                }

        }
        $this->server->httpResponse->setHeader('Location',$this->server->httpRequest->getUri());
        return false;
		
		
		return true;

    }

	/**
	 * Creating a Version-Controlled Resource
	 *
	 * Doc reference: RFC3253 2.2.1
	 *
	 * @param string $uri
	 * @return void
	 */
	protected function createVersionResource($uri){
		
		// 1. Create a new "version history resource"
		/*		Get the following from the Backend:
				this is done by calling from the main repos collection "createDirectory"
				Class implementant ICollection (Implicite INode) + IProperty (Implicite INode)
				
				Method to get a default History resource name, 
					1. Retreive the "getChildren()" from the main Collection, and count how many are exist,
					2. Add 1 to the result and prefix it with "his", this result to a newhistoryname, 
					3. Check if that newhistoryname exist,
					3.1 If true: go to next
					3.2 If false: go back to step 2
					4. call "createDirectory" from the main collection
					
		
		*/
		
		$node = $server->tree->getNodeForPath($uri,0);
		
		if (!($node instanceof Sabre_DAV_IProperties)){
			throw new Sabre_DAV_Exception_NotImplemented('Properties support, on your DAV server, are required by DeltaV. Check your configuration !');
		}
		
		if (!($node instanceof Sabre_DAV_IFile)){
			// TODO: Dispach the event "onCollectionVersionControl" (this is for "14. Version-Controlled-Collection" futur plugin)
			throw new Sabre_DAV_Exception_NotImplemented('VERSION-CONTROL is only implemented on File objects');
		}
		
		// Check if a version Controled resource already exist for the uri
		if(isset($node->getProperties("{DAV:}checked-in")["{DAV:}checked-in"]) || isset($node->getProperties("{DAV:}checked-out")["{DAV:}checked-out"])){
			// This node is already a version-controlled resource, 
			// do nothing, just exit
			return;
		}
		
		// This is an Alternative way [for futur plugin] to check if the ressource are under versioning control (eg. In case of IProperties are not supported)
		if($this->historyManager->isUnderVersioning($uri)){
			return;
		}
		
		
		// Create a new Version History
		$his = $this->historyManager->createHistory();
		$body = $node->get();
		$props = $node->getProperties();
		$ver = $his->createVersion($body, $props);
		
		// Set the "Checkin" property
		$node->updateProperties(array("{DAV:}checked-in" => $ver->uri);
		
		// If a request body is included, it MUST be a DAV:version-control XML element.
		
		
		
		
		
		
		// 2. Create a "version resource"
		//		Call "createFile" from the previous newly created "version history resource" [The name should be automatic]
		//		Get the following from the History Resource:
		//		Class implementant Sabre_DAV_IFile (Implicite INode) +  IProperty (Implicite INode)
		//		Perform a "put"
		//		Perform a "updateProperties" from the getProperties the $uri specified in param
		
		// 3. Create a "version-controlled-resource"
				// Update/add the properties to the $uri and by "DAV:checked-in" property
	}
	
	
	public function checkIn($uri){
		// Perform a checkin Operation
		
		// 1. Check if the Ressource are currently CheckedOut
		if(!$backend->isCheckout()) throw new Exception();
		
		// 2. Dispatch the event "beforeCheckin"
		// 3. Perform the checkin with the backend
		// 4. Dispatch the event "afterCheckin"
		
		
		
		
		/*
		Unrelevant, just for PUT, PROPPATCH
		$autoVersion = $this->deltaVBackend->getAutoVersion();
		
		if($autoVersion == 'DAV:checkout-checkin'){
		
		}*/
	}
	
	public function checkOut($uri){
		// Perform a checkout Operation
	
	}
	
	
	
	
	
	
	
	
	/*
	
	+--------------------+
	| History Collection | --> abstract of Sabre_DAV_Collection
	+--------------------+
	
	+--------------------+
	| Version Collection | --> abstract of Sabre_DAV_Collection
	+--------------------+
	
	*/
	
	
	
	
    /**
     * Escapes a string for html. 
     * 
     * @param string $value 
     * @return void
     */
    public function escapeHTML($value) {

        return htmlspecialchars($value,ENT_QUOTES,'UTF-8');

    }

    /**
     * Generates the html directory index for a given url 
     *
     * @param string $path 
     * @return string 
     */
    public function generateDirectoryIndex($path) {

        $html = "<html>
<head>
  <title>Index for " . $this->escapeHTML($path) . "/ - SabreDAV " . Sabre_DAV_Version::VERSION . "</title>
  <style type=\"text/css\"> body { Font-family: arial}</style>
</head>
<body>
  <h1>Index for " . $this->escapeHTML($path) . "/</h1>
  <table>
    <tr><th>Name</th><th>Type</th><th>Size</th><th>Last modified</th></tr>
    <tr><td colspan=\"4\"><hr /></td></tr>";
    
    $files = $this->server->getPropertiesForPath($path,array(
        '{DAV:}displayname',
        '{DAV:}resourcetype',
        '{DAV:}getcontenttype',
        '{DAV:}getcontentlength',
        '{DAV:}getlastmodified',
    ),1);

    $parent = $this->server->tree->getNodeForPath($path);


    if ($path) {

        list($parentUri) = Sabre_DAV_URLUtil::splitPath($path);
        $fullPath = Sabre_DAV_URLUtil::encodePath($this->server->getBaseUri() . $parentUri);

        $html.= "<tr>
<td><a href=\"{$fullPath}\">..</a></td>
<td>[parent]</td>
<td></td>
<td></td>
</tr>";

    }

    foreach($files as $k=>$file) {

        // This is the current directory, we can skip it
        if (rtrim($file['href'],'/')==$path) continue;

        list(, $name) = Sabre_DAV_URLUtil::splitPath($file['href']);

        $type = null;


        if (isset($file[200]['{DAV:}resourcetype'])) {
            $type = $file[200]['{DAV:}resourcetype']->getValue();

            // resourcetype can have multiple values
            if (!is_array($type)) $type = array($type);

            foreach($type as $k=>$v) { 

                // Some name mapping is preferred 
                switch($v) {
                    case '{DAV:}collection' :
                        $type[$k] = 'Collection';
                        break;
                    case '{DAV:}principal' :
                        $type[$k] = 'Principal';
                        break;
                    case '{urn:ietf:params:xml:ns:carddav}addressbook' :
                        $type[$k] = 'Addressbook';
                        break;
                    case '{urn:ietf:params:xml:ns:caldav}calendar' :
                        $type[$k] = 'Calendar';
                        break;
                }

            }
            $type = implode(', ', $type);
        }

        // If no resourcetype was found, we attempt to use
        // the contenttype property
        if (!$type && isset($file[200]['{DAV:}getcontenttype'])) {
            $type = $file[200]['{DAV:}getcontenttype'];
        }
        if (!$type) $type = 'Unknown';

        $size = isset($file[200]['{DAV:}getcontentlength'])?(int)$file[200]['{DAV:}getcontentlength']:'';
        $lastmodified = isset($file[200]['{DAV:}getlastmodified'])?$file[200]['{DAV:}getlastmodified']->getTime()->format(DateTime::ATOM):'';

        $fullPath = Sabre_DAV_URLUtil::encodePath('/' . trim($this->server->getBaseUri() . ($path?$path . '/':'') . $name,'/'));

        $displayName = isset($file[200]['{DAV:}displayname'])?$file[200]['{DAV:}displayname']:$name;

        $name = $this->escapeHTML($name);
        $displayName = $this->escapeHTML($displayName);
        $type = $this->escapeHTML($type);

        $html.= "<tr>
<td><a href=\"{$fullPath}\">{$displayName}</a></td>
<td>{$type}</td>
<td>{$size}</td>
<td>{$lastmodified}</td>
</tr>";

    }

  $html.= "<tr><td colspan=\"4\"><hr /></td></tr>";

  if ($this->enablePost && $parent instanceof Sabre_DAV_ICollection) {
      $html.= '<tr><td><form method="post" action="">
            <h3>Create new folder</h3>
            <input type="hidden" name="sabreAction" value="mkcol" />
            Name: <input type="text" name="name" /><br />
            <input type="submit" value="create" />
            </form>
            <form method="post" action="" enctype="multipart/form-data">
            <h3>Upload file</h3>
            <input type="hidden" name="sabreAction" value="put" />
            Name (optional): <input type="text" name="name" /><br />
            File: <input type="file" name="file" /><br />
            <input type="submit" value="upload" />
            </form>
       </td></tr>';
  }

  $html.= "</table>
  <address>Generated by SabreDAV " . Sabre_DAV_Version::VERSION ."-". Sabre_DAV_Version::STABILITY . " (c)2007-2011 <a href=\"http://code.google.com/p/sabredav/\">http://code.google.com/p/sabredav/</a></address>
</body>
</html>";

        return $html; 

    }

}
?>